# Copyright (C) 2016 EDF
# All Rights Reserved
# This code is published under the GNU Lesser General Public License (GNU LGPL)
import numpy as np
import math as maths
import BasketOptions as bo
import BlackScholesSimulator as bs
import americanOption as amer
import StOptGrids
import StOptReg as reg
import unittest



def american(p_nDim, p_nbSimul, p_bandwidth) :
    
    initialValues = np.zeros(p_nDim) + 1.0
    sigma = np.zeros(p_nDim) + 0.2
    mu = np.zeros(p_nDim) + 0.05
    corr = np.zeros((p_nDim,p_nDim))
    T = 1.
    nDate = 10
    np.fill_diagonal(corr, 1.)
    strike = 1.

    # simulator
    simulator = bs.BlackScholesSimulator(initialValues, sigma, mu, corr, T, nDate, p_nbSimul, False)
    # payoff
    payoff = bo.BasketPut(strike)
    # proportion of grid points
    prop =1.
    # regressor
    regressor = reg.LocalGridKernelRegression(p_bandwidth, prop)
    # bermudean value
    value = amer.resolution(simulator, payoff, regressor)
    return value

# test cases
class testAmericanOptionKernelTest(unittest.TestCase):
    
    def test_americanBasket1D(self):
  
        # dimension
        nDim = 1
        nbSimul = 500000
        nbMesh = 16
        referenceValue = 0.06031
        accuracyEqual = 0.13
               
        self.assertAlmostEqual(american(nDim, nbSimul, nbMesh), referenceValue, None, None, accuracyEqual)
        
    def test_americanBasket2D(self):    
        
        # dimension
        nDim = 2
        nbSimul = 1000000
        nbMesh = 16
        referenceValue = 0.03882
        accuracyEqual = 0.4
            
        self.assertAlmostEqual(american(nDim, nbSimul, nbMesh), referenceValue, None, None, accuracyEqual)
     

if __name__ == '__main__':
    unittest.main()
